@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import logging
 import os
 import pathlib
@@ -19,7 +21,7 @@ from pydantic_core import CoreSchema, core_schema
 from . import overrides
 
 if typing.TYPE_CHECKING:
-    from . import build_environment
+    from . import build_environment, context
 
 logger = logging.getLogger(__name__)
 
@@ -100,6 +102,54 @@ Variant = typing.NewType("Variant", str)
 GlobalChangelog = Mapping[Variant, list[str]]
 VariantChangelog = Mapping[PackageVersion, list[str]]
 
+# Annotations
+RawAnnotations = Mapping[str, str]
+
+
+class Annotations(Mapping):
+    """Read-only mapping for package annotations"""
+
+    __slots__ = "_mapping"
+
+    def __init__(
+        self,
+        package: RawAnnotations | None = None,
+        variant: RawAnnotations | None = None,
+    ) -> None:
+        self._mapping: RawAnnotations = {}
+        if package:
+            self._mapping.update(package)
+        if variant:
+            self._mapping.update(variant)
+
+    def __getitem__(self, key: str) -> str:
+        return self._mapping[key]
+
+    def __iter__(self) -> typing.Iterator[str]:
+        return iter(self._mapping)
+
+    def __len__(self) -> int:
+        return len(self._mapping)
+
+    def __repr__(self):
+        return repr(self._mapping)
+
+    def getbool(self, key: str) -> bool:
+        """Get bool from string value
+
+        raises :exc:`KeyError` when key is missing and :exc`ValueError` when
+        the value is not 1, true, on, yes, 0, false, off, no.
+        """
+        value = self[key]
+        match value.lower():
+            case "1" | "true" | "on" | "yes":
+                return True
+            case "0" | "false" | "off" | "no":
+                return False
+            case _:
+                raise ValueError(value)
+
+
 # common settings
 MODEL_CONFIG = pydantic.ConfigDict(
     # don't accept unknown keys
@@ -277,6 +327,13 @@ class VariantInfo(pydantic.BaseModel):
 
     model_config = MODEL_CONFIG
 
+    annotations: RawAnnotations | None = None
+    """Arbitrary metadata for variants
+
+    Variant annotation keys have a higher precedence than package
+    annotation keys.
+    """
+
     env: EnvVars = Field(default_factory=dict)
     """Additional env vars (overrides package env vars)"""
 
@@ -360,6 +417,9 @@ class PackageSettings(pydantic.BaseModel):
     has_config: bool
     """package has override setting"""
 
+    annotations: RawAnnotations | None = None
+    """Arbitrary metadata for a package"""
+
     build_dir: BuildDirectory | None = None
     """sub-directory with setup.py or pyproject.toml"""
 
@@ -419,7 +479,7 @@ class PackageSettings(pydantic.BaseModel):
         *,
         source: pathlib.Path | str | None,
         has_config: bool,
-    ) -> "PackageSettings":
+    ) -> PackageSettings:
         """Load from a dict"""
         package = Package(canonicalize_name(package, validate=True))
         try:
@@ -436,7 +496,7 @@ class PackageSettings(pydantic.BaseModel):
         raw_yaml: str,
         *,
         source: pathlib.Path | str | None = None,
-    ) -> "PackageSettings":
+    ) -> PackageSettings:
         """Load from raw yaml string"""
         parsed: typing.Any = yaml.safe_load(raw_yaml)
         if parsed is None:
@@ -449,7 +509,7 @@ class PackageSettings(pydantic.BaseModel):
         return cls.from_mapping(package, parsed, source=source, has_config=True)
 
     @classmethod
-    def from_file(cls, filename: pathlib.Path) -> "PackageSettings":
+    def from_file(cls, filename: pathlib.Path) -> PackageSettings:
         """Load from file
 
         Raises :exc:`FileNotFound` when the file is not found.
@@ -461,7 +521,7 @@ class PackageSettings(pydantic.BaseModel):
         return cls.from_string(filename.stem, raw_yaml, source=filename)
 
     @classmethod
-    def from_default(cls, package: str | Package) -> "PackageSettings":
+    def from_default(cls, package: str | Package) -> PackageSettings:
         """Create a default package setting"""
         return cls.from_mapping(package, {}, source="default", has_config=False)
 
@@ -558,7 +618,7 @@ class PackageBuildInfo:
     Public API for PackageSettings with i
     """
 
-    def __init__(self, settings: "Settings", ps: PackageSettings) -> None:
+    def __init__(self, settings: Settings, ps: PackageSettings) -> None:
         self._variant = typing.cast(Variant, settings.variant)
         self._patches_dir = settings.patches_dir
         self._variant_changelog = settings.variant_changelog()
@@ -566,6 +626,7 @@ class PackageBuildInfo:
         self._ps = ps
         self._plugin_module: types.ModuleType | None | typing.Literal[False] = False
         self._patches: PatchMap | None = None
+        self._annotations: Annotations | None = None
 
     @property
     def package(self) -> NormalizedName:
@@ -577,6 +638,31 @@ class PackageBuildInfo:
         """Variant name"""
         return self._variant
 
+    @property
+    def annotations(self) -> Annotations:
+        """Get Package and variant annotations
+
+        Annotations can be used to attach arbitrary metadata to packages and
+        package variants. The feature is inspired by Kubernetes's
+        annotations. Variant keys have a higher precedence than package keys.
+
+        The prefix ``fromager.`` is reserved for future use by Fromager.
+
+        ::
+
+           annotations:
+             "downstream.maintainer": "Platform Team"
+           variants:
+             cuda:
+               annotations:
+                 "downstream.maintainer": "CUDA Accelerator Team"
+        """
+        if self._annotations is None:
+            vi = self._ps.variants.get(self.variant)
+            va = vi.annotations if vi is not None else None
+            self._annotations = Annotations(self._ps.annotations, va)
+        return self._annotations
+
     @property
     def plugin(self) -> types.ModuleType | None:
         """Get Fromager plugin module"""
@@ -763,7 +849,7 @@ class PackageBuildInfo:
         self,
         *,
         template_env: dict[str, str] | None = None,
-        build_env: "build_environment.BuildEnvironment | None" = None,
+        build_env: build_environment.BuildEnvironment | None = None,
     ) -> dict[str, str]:
         """Get extra environment variables for a variant
 
@@ -894,7 +980,7 @@ class SettingsFile(pydantic.BaseModel):
         raw_yaml: str,
         *,
         source: pathlib.Path | str | None = None,
-    ) -> "SettingsFile":
+    ) -> SettingsFile:
         """Load from raw yaml string"""
         parsed: typing.Any = yaml.safe_load(raw_yaml)
         if parsed is None:
@@ -916,7 +1002,7 @@ class SettingsFile(pydantic.BaseModel):
             ) from err
 
     @classmethod
-    def from_file(cls, filename: pathlib.Path) -> "SettingsFile":
+    def from_file(cls, filename: pathlib.Path) -> SettingsFile:
         """Load from file
 
         Raises :exc:`FileNotFound` when the file is not found.
@@ -958,7 +1044,7 @@ class Settings:
         variant: Variant | str,
         patches_dir: pathlib.Path,
         max_jobs: int | None,
-    ) -> "Settings":
+    ) -> Settings:
         """Create Settings from settings.yaml and directory"""
         if settings_file.is_file():
             settings = SettingsFile.from_file(settings_file)
@@ -1082,3 +1168,41 @@ class Settings:
         for ps in self._package_settings.values():
             variants.update(ps.variants.keys())
         return variants
+
+
+def default_update_extra_environ(
+    *,
+    ctx: context.WorkContext,
+    req: Requirement,
+    version: Version | None,
+    sdist_root_dir: pathlib.Path,
+    extra_environ: dict[str, str],
+    build_env: build_environment.BuildEnvironment,
+) -> None:
+    """Update extra_environ in-place"""
+    return None
+
+
+def get_extra_environ(
+    *,
+    ctx: context.WorkContext,
+    req: Requirement,
+    version: Version | None,
+    sdist_root_dir: pathlib.Path,
+    build_env: build_environment.BuildEnvironment,
+) -> dict[str, str]:
+    """Get extra environment variables from settings and update hook"""
+    pbi = ctx.package_build_info(req)
+    extra_environ = pbi.get_extra_environ(build_env=build_env)
+    overrides.find_and_invoke(
+        req.name,
+        "update_extra_environ",
+        default_update_extra_environ,
+        ctx=ctx,
+        req=req,
+        version=version,
+        sdist_root_dir=sdist_root_dir,
+        extra_environ=extra_environ,
+        build_env=build_env,
+    )
+    return extra_environ
