@@ -103,6 +103,13 @@ def default_resolver_provider(
     )
 
 
+def extract_filename_from_url(url: str) -> str:
+    """Extract filename from URL and decode it."""
+    path = urlparse(url).path
+    filename = os.path.basename(path)
+    return unquote(filename)
+
+
 class LogReporter(resolvelib.BaseReporter):
     """Report resolution events
 
@@ -184,13 +191,26 @@ def get_project_from_pypi(
         dist_info_metadata = i.attrib.get("data-core-metadata") or i.attrib.get(
             "data-dist-info-metadata"
         )
-        path = urlparse(candidate_url).path
+        # PEP 592: Check if package was yanked
+        reason_data_yanked = i.attrib.get("data-yanked")
         # file names are URL quoted, "1.0%2Blocal" -> "1.0+local"
-        filename = unquote(path.rsplit("/", 1)[-1])
+        filename = extract_filename_from_url(candidate_url)
         found_candidates.add(filename)
         if DEBUG_RESOLVER:
             logger.debug("%s: candidate %r -> %r", project, candidate_url, filename)
 
+        # PEP 592: Skip items that were yanked
+        if reason_data_yanked is not None:
+            if DEBUG_RESOLVER:
+                logger.debug(
+                    "%s: skipping %s because it was yanked (%s)",
+                    project,
+                    filename,
+                    reason_data_yanked if reason_data_yanked else "no reason found",
+                )
+            ignored_candidates.add(filename)
+            continue
+
         # Construct metadata URL if PEP 658 metadata is available
         metadata_url = None
         if dist_info_metadata:
