@@ -20,9 +20,11 @@ from packaging.utils import BuildTag, canonicalize_name, parse_wheel_filename
 from packaging.version import Version
 
 from . import (
+    dependencies,
     external_commands,
     metrics,
     overrides,
+    packagesettings,
     requirements_file,
     resolver,
     sources,
@@ -53,6 +55,7 @@ def _extra_metadata_elfdeps(
     # mapping of required libraries to list of versions
     requires: set[elfdeps.SOInfo] = set()
     provides: set[elfdeps.SOInfo] = set()
+    runpaths: set[str] = set()
     elfinfos: list[elfdeps.ELFInfo] = []
 
     settings = elfdeps.ELFAnalyzeSettings(filter_soname=True)
@@ -68,6 +71,8 @@ def _extra_metadata_elfdeps(
         )
         provides.update(info.provides)
         requires.update(info.requires)
+        if info.runpath is not None:
+            runpaths.update(info.runpath)
         elfinfos.append(info)
 
     # Don't list provided names as requirements
@@ -103,6 +108,9 @@ def _extra_metadata_elfdeps(
             for soinfo in sorted(provides):
                 f.write(f"{soinfo}\n")
 
+    if runpaths:
+        logger.info("Libraries have runpath: %s", " ".join(sorted(runpaths)))
+
     return elfinfos
 
 
@@ -269,7 +277,13 @@ def build_wheel(
 
     # add package and variant env vars, package's parallel job vars, and
     # build_env's virtual env vars.
-    extra_environ = pbi.get_extra_environ(build_env=build_env)
+    extra_environ = packagesettings.get_extra_environ(
+        ctx=ctx,
+        req=req,
+        version=version,
+        sdist_root_dir=sdist_root_dir,
+        build_env=build_env,
+    )
 
     if (
         pbi.build_ext_parallel
@@ -308,6 +322,9 @@ def build_wheel(
             f"Expected 1 built wheel in {ctx.wheels_build}, got {len(wheels)}"
         )
 
+    # invalidate uv's cache
+    build_env.clean_cache(req)
+
     wheel = add_extra_metadata_to_wheels(
         ctx=ctx,
         req=req,
@@ -319,6 +336,37 @@ def build_wheel(
     return wheel
 
 
+def pep517_build_wheel(
+    ctx: context.WorkContext,
+    build_env: build_environment.BuildEnvironment,
+    extra_environ: dict[str, str],
+    req: Requirement,
+    sdist_root_dir: pathlib.Path,
+    version: Version,
+    build_dir: pathlib.Path,
+) -> pathlib.Path:
+    """Use the PEP 517 API to build a wheel distribution"""
+    logger.debug(f"building wheel in {build_dir} with {extra_environ}")
+
+    hook_caller = dependencies.get_build_backend_hook_caller(
+        ctx=ctx,
+        req=req,
+        build_dir=build_dir,
+        override_environ=extra_environ,
+        build_env=build_env,
+        log_filename=str(sdist_root_dir.parent / "build.log"),
+    )
+
+    pbi = ctx.package_build_info(req)
+    wheel_filename = hook_caller.build_wheel(
+        str(ctx.wheels_build),
+        config_settings=pbi.config_settings,
+    )
+    logger.debug("built wheel %s", wheel_filename)
+
+    return ctx.wheels_build / wheel_filename
+
+
 def default_build_wheel(
     ctx: context.WorkContext,
     build_env: build_environment.BuildEnvironment,
@@ -328,42 +376,15 @@ def default_build_wheel(
     version: Version,
     build_dir: pathlib.Path,
 ) -> None:
-    logger.debug(f"building wheel in {build_dir} with {extra_environ}")
-    pbi = ctx.package_build_info(req)
-
-    cmd = [
-        os.fspath(build_env.python),
-        "-m",
-        "pip",
-        "-vvv",
-        "--disable-pip-version-check",
-        "wheel",
-        "--no-build-isolation",
-        "--only-binary",
-        ":all:",
-        "--wheel-dir",
-        os.fspath(ctx.wheels_build),
-        "--no-deps",
-        "--index-url",
-        ctx.wheel_server_url,  # probably redundant, but just in case
-        "--log",
-        os.fspath(build_dir.parent / "build.log"),
-    ]
-    # config settings needs pip >= 24.0 to work. Fromager uses `virtualenv``
-    # package to create virtual envs, which comes with recent pip. Stdlib's
-    # `venv` comes with rather old pip.
-    for key, values in pbi.config_settings.items():
-        for value in values:
-            cmd.append(f"--config-settings={key}={value}")
-    cmd.append(os.fspath(build_dir))
-
-    with tempfile.TemporaryDirectory() as dir_name:
-        build_env.run(
-            cmd,
-            cwd=dir_name,
-            extra_environ=extra_environ,
-            network_isolation=ctx.network_isolation,
-        )
+    pep517_build_wheel(
+        ctx=ctx,
+        build_env=build_env,
+        extra_environ=extra_environ,
+        req=req,
+        sdist_root_dir=sdist_root_dir,
+        version=version,
+        build_dir=build_dir,
+    )
 
 
 def download_wheel(
